//+------------------------------------------------------------------+
//|                                               Phantom_AddSix_02a |
//+------------------------------------------------------------------+
/*
Attach to any chart, any timeframe. Do not attach to more than one
chart.

These are the pairs we trade:

For InterbankFX
---------------
Group 1, The Phantom Buys
1.  GBPUSD
2.  EURGBP
3.  GBPJPY
4.  USDCHF
5.  NZDUSD
6.  AUDJPY
7.  EURJPY

Group 2, The Phantom Sells
8.  EURUSD
9.  USDJPY
10. AUDUSD
11. NZDJPY
12. GBPCHF
13. CHFJPY
14. EURCHF

Initially, buy the phantom buy group of pairs and sell the phantom-sell
group of pairs.

Every hour at minute 0, add a trade to the biggest winning phantom pair
if the pair is one of EURCHF, EURJPY, CHFJPY, GBPCHF, GBPUSD, USDCHF.
Reset the phantom trades every 8 hours.

Close all when current equity exceeds 1% of last flat equity.

*/

#include <Trade\Trade.mqh>

CTrade mytrade;

//+------------------------------------------------------------------+
//|              External Variables                                  |
//+------------------------------------------------------------------+

input bool   Bailout = false;

input int    Slippage         =       3;
input int    MiniSize         =   10000;
input int    StdSize          =  100000;

input int GBPUSDidx = 0;
input int EURGBPidx = 1;
input int GBPJPYidx = 2;
input int USDCHFidx = 3;
input int NZDUSDidx = 4;
input int AUDJPYidx = 5;
input int EURJPYidx = 6;

input int EURUSDidx = 7;
input int USDJPYidx = 8;
input int AUDUSDidx = 9;
input int NZDJPYidx = 10;
input int GBPCHFidx = 11;
input int CHFJPYidx = 12;
input int EURCHFidx = 13;

input int ResetHourInterval = 8;

input bool   UserTradingAllowed   = true;
input bool   StopAfterNoTrades    = false;
input bool   UseSmallestLot       = false;

input double MaxMarginToUse      =     0.3; // Maximum percent of margin to commit to each trade.
input double Lots                =     0.1;
input double ProfitPct           =     0.25;
input double MaxMiniProfit       =  1000.0;
input double MaxStdProfit        = 10000.0;

input int    MagicNumber         =  777111;


//+------------------------------------------------------------------+
//|              Internal Variables                                  |
//+------------------------------------------------------------------+

int i;
int OrderSendResult;
int Leverage;
int NumBuys, NumSells;
int TotalTrades;
int AccountTypeVal;

int GBPUSDbuys, GBPUSDsells;
int EURGBPbuys, EURGBPsells;
int GBPJPYbuys, GBPJPYsells;
int USDCHFbuys, USDCHFsells;
int NZDUSDbuys, NZDUSDsells;
int AUDJPYbuys, AUDJPYsells;
int EURJPYbuys, EURJPYsells;

int EURUSDbuys, EURUSDsells;
int USDJPYbuys, USDJPYsells;
int AUDUSDbuys, AUDUSDsells;
int NZDJPYbuys, NZDJPYsells;
int GBPCHFbuys, GBPCHFsells;
int CHFJPYbuys, CHFJPYsells;
int EURCHFbuys, EURCHFsells;

bool AccountIsMini;
bool CloseAll;
bool BidsHaveChanged = false;
bool PhantomsReset = false, SymbolMarksCleared = false;

string AccountTypeString;
string LastFlatBal;           double valLastFlatBal;
string LastFlatEq;            double valLastFlatEq;
string LowestMarginLevel;     double valLowestMarginLevel;
string LargestFloatingLoss;   double valLargestFloatingLoss;
string MaxFloatingDrawdown;   double valMaxFloatingDrawdown;

string GBPUSDadded; bool valGBPUSDadded;
string EURGBPadded; bool valEURGBPadded;
string GBPJPYadded; bool valGBPJPYadded;
string USDCHFadded; bool valUSDCHFadded;
string NZDUSDadded; bool valNZDUSDadded;
string AUDJPYadded; bool valAUDJPYadded;
string EURJPYadded; bool valEURJPYadded;

string EURUSDadded; bool valEURUSDadded;
string USDJPYadded; bool valUSDJPYadded;
string AUDUSDadded; bool valAUDUSDadded;
string NZDJPYadded; bool valNZDJPYadded;
string GBPCHFadded; bool valGBPCHFadded;
string CHFJPYadded; bool valCHFJPYadded;
string EURCHFadded; bool valEURCHFadded;

string GBPUSDsym;
string EURGBPsym;
string GBPJPYsym;
string USDCHFsym;
string NZDUSDsym;
string AUDJPYsym;
string EURJPYsym;

string EURUSDsym;
string USDJPYsym;
string AUDUSDsym;
string NZDJPYsym;
string GBPCHFsym;
string CHFJPYsym;
string EURCHFsym;

string CommentString;

string SymbolArray[14];

double MaxLots, MinLots, LotStep;
double UseLots, TotalLots, BuyLots, SellLots;
double phantomEAPL, phantomBuyPL, phantomSellPL, PairOverallPL, EAPL, BuyPL, SellPL, EquityExit;
double PercentMarginLevel;
double DollarProfitTarget;
double MaxMarginVar;
double Investment;

// PL for real trades

double GBPUSDPL;
double EURGBPPL;
double GBPJPYPL;
double USDCHFPL;
double NZDUSDPL;
double AUDJPYPL;
double EURJPYPL;

double EURUSDPL;
double USDJPYPL;
double AUDUSDPL;
double NZDJPYPL;
double GBPCHFPL;
double CHFJPYPL;
double EURCHFPL;

// PL for phantom trades

double GBPUSD_PL;
double EURGBP_PL;
double GBPJPY_PL;
double USDCHF_PL;
double NZDUSD_PL;
double AUDJPY_PL;
double EURJPY_PL;

double EURUSD_PL;
double USDJPY_PL;
double AUDUSD_PL;
double NZDJPY_PL;
double GBPCHF_PL;
double CHFJPY_PL;
double EURCHF_PL;

double prevGBPUSDbid = 0.0;
double prevEURGBPbid = 0.0;
double prevGBPJPYbid = 0.0;
double prevUSDCHFbid = 0.0;
double prevNZDUSDbid = 0.0;
double prevAUDJPYbid = 0.0;
double prevEURJPYbid = 0.0;

double prevEURUSDbid = 0.0;
double prevUSDJPYbid = 0.0;
double prevAUDUSDbid = 0.0;
double prevNZDJPYbid = 0.0;
double prevGBPCHFbid = 0.0;
double prevCHFJPYbid = 0.0;
double prevEURCHFbid = 0.0;

double BiggestWinnerPL;
double BiggestWinnerLots;
int    BiggestWinnerTicket;
string BiggestWinnerType;
string BiggestWinnerSymbol;

double BiggestLoserPL;
double BiggestLoserLots;
int    BiggestLoserTicket;
string BiggestLoserType;
string BiggestLoserSymbol;

double BiggestPhantomWinnerPL;
string BiggestPhantomWinnerType;
string BiggestPhantomWinnerSymbol;

string strGBPUSDask; double GBPUSDask = 0.0, GBPUSDpoint, GBPUSDtickval;
string strEURGBPask; double EURGBPask = 0.0, EURGBPpoint, EURGBPtickval;
string strGBPJPYask; double GBPJPYask = 0.0, GBPJPYpoint, GBPJPYtickval;
string strUSDCHFask; double USDCHFask = 0.0, USDCHFpoint, USDCHFtickval;
string strNZDUSDask; double NZDUSDask = 0.0, NZDUSDpoint, NZDUSDtickval;
string strAUDJPYask; double AUDJPYask = 0.0, AUDJPYpoint, AUDJPYtickval;
string strEURJPYask; double EURJPYask = 0.0, EURJPYpoint, EURJPYtickval;

string strEURUSDbid; double EURUSDbid = 0.0, EURUSDpoint, EURUSDtickval;
string strUSDJPYbid; double USDJPYbid = 0.0, USDJPYpoint, USDJPYtickval;
string strAUDUSDbid; double AUDUSDbid = 0.0, AUDUSDpoint, AUDUSDtickval;
string strNZDJPYbid; double NZDJPYbid = 0.0, NZDJPYpoint, NZDJPYtickval;
string strGBPCHFbid; double GBPCHFbid = 0.0, GBPCHFpoint, GBPCHFtickval;
string strCHFJPYbid; double CHFJPYbid = 0.0, CHFJPYpoint, CHFJPYtickval;
string strEURCHFbid; double EURCHFbid = 0.0, EURCHFpoint, EURCHFtickval;

double PL_Array[14];
double TickvalArray[14];

//+------------------------------------------------------------------+
//|   Expert Advisor Initialization, executes only when the EA is    |
//|   first attached to a chart or on platform  restart.             |
//+------------------------------------------------------------------+

int OnInit() //cambiaod por compatibilidad mql5
{
  SetGlobalVariableNames();
  InitGlobalVars();
  ResetPhantoms();

  return (0);
}

//+------------------------------------------------------------------+
//|    Expert Advisor start function, executes on each tick of the   |
//|    currecy pair it's attached to.                                |
//+------------------------------------------------------------------+


void OnTick()
{

 if (Bailout)
  {
    Comment("Expert is not enabled");

    return;
  }

  RefreshGlobalVars();
  GetAccountInfo();  
  SetSymbols();
  SetAccountType();
  SetLotSize();  
  Count_PL_Trades_Lots();
  SetExitTargets();

  BidsHaveChanged = 

    prevGBPUSDbid != 0.0 &&
    prevEURGBPbid != 0.0 &&
    prevGBPJPYbid != 0.0 &&
    prevUSDCHFbid != 0.0 &&
    prevNZDUSDbid != 0.0 &&
    prevAUDJPYbid != 0.0 &&
    prevEURJPYbid != 0.0 &&

    prevEURUSDbid != 0.0 &&
    prevUSDJPYbid != 0.0 &&
    prevAUDUSDbid != 0.0 &&
    prevNZDJPYbid != 0.0 &&
    prevGBPCHFbid != 0.0 &&
    prevCHFJPYbid != 0.0 &&
    prevEURCHFbid != 0.0 &&

    (prevGBPUSDbid != SymbolInfoDouble(GBPUSDsym,SYMBOL_BID) || //era prevGBPUSDbid != MarketInfo(GBPUSDsym,MODE_BID)
     prevEURGBPbid != SymbolInfoDouble(EURGBPsym,SYMBOL_BID) ||
     prevGBPJPYbid != SymbolInfoDouble(GBPJPYsym,SYMBOL_BID) ||
     prevUSDCHFbid != SymbolInfoDouble(USDCHFsym,SYMBOL_BID) ||
     prevNZDUSDbid != SymbolInfoDouble(NZDUSDsym,SYMBOL_BID) ||
     prevAUDJPYbid != SymbolInfoDouble(AUDJPYsym,SYMBOL_BID) ||
     prevEURJPYbid != SymbolInfoDouble(EURJPYsym,SYMBOL_BID) ||
     prevEURUSDbid != SymbolInfoDouble(EURUSDsym,SYMBOL_BID) ||
     prevUSDJPYbid != SymbolInfoDouble(USDJPYsym,SYMBOL_BID) ||
     prevAUDUSDbid != SymbolInfoDouble(AUDUSDsym,SYMBOL_BID) ||
     prevNZDJPYbid != SymbolInfoDouble(NZDJPYsym,SYMBOL_BID) ||
     prevGBPCHFbid != SymbolInfoDouble(GBPCHFsym,SYMBOL_BID) ||
     prevCHFJPYbid != SymbolInfoDouble(CHFJPYsym,SYMBOL_BID) ||
     prevEURCHFbid != SymbolInfoDouble(EURCHFsym,SYMBOL_BID));

  CloseAllTradesIfNeeded();
  OpenPhantomTrades();
  CalcPhantomPL();
  AddIfNeeded();

  if (TotalTrades == 0)
  {
    if (AccountInfoDouble(ACCOUNT_BALANCE) > valLastFlatBal)    //era if (AccountBalance() > valLastFlatBal)
    {
      valLastFlatBal = AccountInfoDouble(ACCOUNT_BALANCE); // era valLastFlatBal = AccountBalance();
      GlobalVariableSet(LastFlatBal, valLastFlatBal);
    }

    if (AccountInfoDouble(ACCOUNT_EQUITY) > valLastFlatEq)  //era if (AccountEquity() > valLastFlatEq)
    {
      valLastFlatEq = AccountInfoDouble(ACCOUNT_EQUITY); //era valLastFlatEq = AccountEquity();
      GlobalVariableSet(LastFlatEq,  valLastFlatEq);
    }

    if (StopAfterNoTrades) return; //era if (!IsExpertEnabled() || StopAfterNoTrades) return(0);

    CloseAll = false;
  }

  if (!CloseAll && valLastFlatEq != 0.0 && AccountInfoDouble(ACCOUNT_EQUITY)-valLastFlatEq > DollarProfitTarget)
  {
    CloseAll = true;
    ResetPhantoms();

    return;
  }

  ExamineMarginAndFloat();

  prevGBPUSDbid = SymbolInfoDouble(GBPUSDsym, SYMBOL_BID); //era prevGBPUSDbid = MarketInfo(GBPUSDsym, MODE_BID);
  prevEURGBPbid = SymbolInfoDouble(EURGBPsym, SYMBOL_BID); //era prevEURGBPbid = MarketInfo(EURGBPsym, MODE_BID);
  prevGBPJPYbid = SymbolInfoDouble(GBPJPYsym, SYMBOL_BID); //era prevGBPJPYbid = MarketInfo(GBPJPYsym, MODE_BID);
  prevUSDCHFbid = SymbolInfoDouble(USDCHFsym, SYMBOL_BID); //era prevUSDCHFbid = MarketInfo(USDCHFsym, MODE_BID);
  prevNZDUSDbid = SymbolInfoDouble(NZDUSDsym, SYMBOL_BID); //era prevNZDUSDbid = MarketInfo(NZDUSDsym, MODE_BID);
  prevAUDJPYbid = SymbolInfoDouble(AUDJPYsym, SYMBOL_BID); //era prevAUDJPYbid = MarketInfo(AUDJPYsym, MODE_BID);
  prevEURJPYbid = SymbolInfoDouble(EURJPYsym, SYMBOL_BID); //era prevEURJPYbid = MarketInfo(EURJPYsym, MODE_BID);

  prevEURUSDbid = SymbolInfoDouble(EURUSDsym, SYMBOL_BID); //era prevEURUSDbid = MarketInfo(EURUSDsym, MODE_BID);
  prevUSDJPYbid = SymbolInfoDouble(USDJPYsym, SYMBOL_BID); //era prevUSDJPYbid = MarketInfo(USDJPYsym, MODE_BID);
  prevAUDUSDbid = SymbolInfoDouble(AUDUSDsym, SYMBOL_BID); //era prevAUDUSDbid = MarketInfo(AUDUSDsym, MODE_BID);
  prevNZDJPYbid = SymbolInfoDouble(NZDJPYsym, SYMBOL_BID); //era prevNZDJPYbid = MarketInfo(NZDJPYsym, MODE_BID);
  prevGBPCHFbid = SymbolInfoDouble(GBPCHFsym, SYMBOL_BID); //era prevGBPCHFbid = MarketInfo(GBPCHFsym, MODE_BID);
  prevCHFJPYbid = SymbolInfoDouble(CHFJPYsym, SYMBOL_BID); //era prevCHFJPYbid = MarketInfo(CHFJPYsym, MODE_BID);
  prevEURCHFbid = SymbolInfoDouble(EURCHFsym, SYMBOL_BID); //era prevEURCHFbid = MarketInfo(EURCHFsym, MODE_BID);

  PrintChartComments();
    
  return;
}

//+------------------------------------------------------------------+
//|          Assign values to global variable namestrings.           |
//+------------------------------------------------------------------+

void SetGlobalVariableNames()
{
  LowestMarginLevel = IntegerToString(MagicNumber)+"_LowestMarginLevel";
  LargestFloatingLoss = IntegerToString(MagicNumber)+"_LargestFloatingLoss";
  MaxFloatingDrawdown = IntegerToString(MagicNumber)+"_MaxFloatingDrawdown";

  LastFlatBal = IntegerToString(MagicNumber)+"_LastFlatBal";
  LastFlatEq  = IntegerToString(MagicNumber)+"_LastFlatEq";

  GBPUSDadded = IntegerToString(MagicNumber)+"_GBPUSDadded";
  EURGBPadded = IntegerToString(MagicNumber)+"_EURGBPadded";
  GBPJPYadded = IntegerToString(MagicNumber)+"_GBPJPYadded";
  USDCHFadded = IntegerToString(MagicNumber)+"_USDCHFadded";
  NZDUSDadded = IntegerToString(MagicNumber)+"_NZDUSDadded";
  AUDJPYadded = IntegerToString(MagicNumber)+"_AUDJPYadded";
  EURJPYadded = IntegerToString(MagicNumber)+"_EURJPYadded";

  EURUSDadded = IntegerToString(MagicNumber)+"_EURUSDadded";
  USDJPYadded = IntegerToString(MagicNumber)+"_USDJPYadded";
  AUDUSDadded = IntegerToString(MagicNumber)+"_AUDUSDadded";
  NZDJPYadded = IntegerToString(MagicNumber)+"_NZDJPYadded";
  GBPCHFadded = IntegerToString(MagicNumber)+"_GBPCHFadded";
  CHFJPYadded = IntegerToString(MagicNumber)+"_CHFJPYadded";
  EURCHFadded = IntegerToString(MagicNumber)+"_EURCHFadded";

  return; //era return(0);
}

//+------------------------------------------------------------------+
//|            Assign initial values to global variables.            |
//+------------------------------------------------------------------+

void InitGlobalVars()
{
  if (!GlobalVariableCheck(LowestMarginLevel)) GlobalVariableSet(LowestMarginLevel, 100000000.0);
  if (!GlobalVariableCheck(LargestFloatingLoss)) GlobalVariableSet(LargestFloatingLoss, 0.0);
  if (!GlobalVariableCheck(MaxFloatingDrawdown)) GlobalVariableSet(MaxFloatingDrawdown, 0.0);

  if (!GlobalVariableCheck(LastFlatBal)) GlobalVariableSet(LastFlatBal, AccountInfoDouble(ACCOUNT_BALANCE));  //era if (!GlobalVariableCheck(LastFlatBal)) GlobalVariableSet(LastFlatBal, AccountBalance());
  if (!GlobalVariableCheck(LastFlatEq)) GlobalVariableSet(LastFlatEq, AccountInfoDouble(ACCOUNT_EQUITY)); //era if (!GlobalVariableCheck(LastFlatEq)) GlobalVariableSet(LastFlatEq, AccountEquity());

  if (!GlobalVariableCheck(GBPUSDadded)) GlobalVariableSet(GBPUSDadded, false);
  if (!GlobalVariableCheck(EURGBPadded)) GlobalVariableSet(EURGBPadded, false);
  if (!GlobalVariableCheck(GBPJPYadded)) GlobalVariableSet(GBPJPYadded, false);
  if (!GlobalVariableCheck(USDCHFadded)) GlobalVariableSet(USDCHFadded, false);
  if (!GlobalVariableCheck(NZDUSDadded)) GlobalVariableSet(NZDUSDadded, false);
  if (!GlobalVariableCheck(AUDJPYadded)) GlobalVariableSet(AUDJPYadded, false);
  if (!GlobalVariableCheck(EURJPYadded)) GlobalVariableSet(EURJPYadded, false);

  if (!GlobalVariableCheck(EURUSDadded)) GlobalVariableSet(EURUSDadded, false);
  if (!GlobalVariableCheck(USDJPYadded)) GlobalVariableSet(USDJPYadded, false);
  if (!GlobalVariableCheck(AUDUSDadded)) GlobalVariableSet(AUDUSDadded, false);
  if (!GlobalVariableCheck(NZDJPYadded)) GlobalVariableSet(NZDJPYadded, false);
  if (!GlobalVariableCheck(GBPCHFadded)) GlobalVariableSet(GBPCHFadded, false);
  if (!GlobalVariableCheck(CHFJPYadded)) GlobalVariableSet(CHFJPYadded, false);
  if (!GlobalVariableCheck(EURCHFadded)) GlobalVariableSet(EURCHFadded, false);

  return; //era return(0);
}

//+------------------------------------------------------------------+
//|   Get global variable values via their namestring and assign the |
//|   stored values to the variables used in the EA's logic.         |
//+------------------------------------------------------------------+

void RefreshGlobalVars()
{
  valLowestMarginLevel = GlobalVariableGet(LowestMarginLevel);
  valLargestFloatingLoss = GlobalVariableGet(LargestFloatingLoss);
  valMaxFloatingDrawdown = GlobalVariableGet(MaxFloatingDrawdown);

  valLastFlatBal = GlobalVariableGet(LastFlatBal);
  valLastFlatEq  = GlobalVariableGet(LastFlatEq);

  valGBPUSDadded = GlobalVariableGet(GBPUSDadded);
  valEURGBPadded = GlobalVariableGet(EURGBPadded);
  valGBPJPYadded = GlobalVariableGet(GBPJPYadded);
  valUSDCHFadded = GlobalVariableGet(USDCHFadded);
  valNZDUSDadded = GlobalVariableGet(NZDUSDadded);
  valAUDJPYadded = GlobalVariableGet(AUDJPYadded);
  valEURJPYadded = GlobalVariableGet(EURJPYadded);

  valEURUSDadded = GlobalVariableGet(EURUSDadded);
  valUSDJPYadded = GlobalVariableGet(USDJPYadded);
  valAUDUSDadded = GlobalVariableGet(AUDUSDadded);
  valNZDJPYadded = GlobalVariableGet(NZDJPYadded);
  valGBPCHFadded = GlobalVariableGet(GBPCHFadded);
  valCHFJPYadded = GlobalVariableGet(CHFJPYadded);
  valEURCHFadded = GlobalVariableGet(EURCHFadded);

  return; //era return(0);
}

//+------------------------------------------------------------------+
//|  Get important account parameters that affect how the lot size   |
//|  will be set.                                                    |
//+------------------------------------------------------------------+

void GetAccountInfo()
{
  // Get account leverage, account type.
  Leverage       = (int)AccountInfoInteger(ACCOUNT_LEVERAGE);

  AccountTypeVal = (int)SymbolInfoDouble(Symbol(), SYMBOL_TRADE_CONTRACT_SIZE);
  MaxLots        = SymbolInfoDouble(Symbol(), SYMBOL_VOLUME_MAX);
  MinLots        = SymbolInfoDouble(Symbol(), SYMBOL_VOLUME_MIN);
  LotStep        = SymbolInfoDouble(Symbol(), SYMBOL_VOLUME_STEP);

  return; // era return(0);
}

//+------------------------------------------------------------------+
//|                 Set the traded symbol strings.                   |
//+------------------------------------------------------------------+

void SetSymbols()
{
  string AddMangle = "";
  if (StringLen(Symbol()) > 6) AddMangle = StringSubstr(Symbol(), 6);

  GBPUSDsym = "GBPUSD" + AddMangle;
  EURGBPsym = "EURGBP" + AddMangle;
  GBPJPYsym = "GBPJPY" + AddMangle;
  USDCHFsym = "USDCHF" + AddMangle;
  NZDUSDsym = "NZDUSD" + AddMangle;
  AUDJPYsym = "AUDJPY" + AddMangle;
  EURJPYsym = "EURJPY" + AddMangle;

  EURUSDsym = "EURUSD" + AddMangle;
  USDJPYsym = "USDJPY" + AddMangle;
  AUDUSDsym = "AUDUSD" + AddMangle;
  NZDJPYsym = "NZDJPY" + AddMangle;
  GBPCHFsym = "GBPCHF" + AddMangle;
  CHFJPYsym = "CHFJPY" + AddMangle;
  EURCHFsym = "EURCHF" + AddMangle;

  SymbolArray[GBPUSDidx] = GBPUSDsym;
  SymbolArray[EURGBPidx] = EURGBPsym;
  SymbolArray[GBPJPYidx] = GBPJPYsym;
  SymbolArray[USDCHFidx] = USDCHFsym;
  SymbolArray[NZDUSDidx] = NZDUSDsym;
  SymbolArray[AUDJPYidx] = AUDJPYsym;
  SymbolArray[EURJPYidx] = EURJPYsym;

  SymbolArray[EURUSDidx] = EURUSDsym;
  SymbolArray[USDJPYidx] = USDJPYsym;
  SymbolArray[AUDUSDidx] = AUDUSDsym;
  SymbolArray[NZDJPYidx] = NZDJPYsym;
  SymbolArray[GBPCHFidx] = GBPCHFsym;
  SymbolArray[CHFJPYidx] = CHFJPYsym;
  SymbolArray[EURCHFidx] = EURCHFsym;

  return; //era return(0);
}

//+------------------------------------------------------------------+
//|                    Reset the phantom trades.                     |
//+------------------------------------------------------------------+

void ResetPhantoms()
{
  GBPUSDask = 0.0;
  EURGBPask = 0.0;
  GBPJPYask = 0.0;
  USDCHFask = 0.0;
  NZDUSDask = 0.0;
  AUDJPYask = 0.0;
  EURJPYask = 0.0;

  EURUSDbid = 0.0;
  USDJPYbid = 0.0;
  AUDUSDbid = 0.0;
  NZDJPYbid = 0.0;
  GBPCHFbid = 0.0;
  CHFJPYbid = 0.0;
  EURCHFbid = 0.0;

  GlobalVariableSet(strGBPUSDask, GBPUSDask);
  GlobalVariableSet(strEURGBPask, EURGBPask);
  GlobalVariableSet(strGBPJPYask, GBPJPYask);
  GlobalVariableSet(strUSDCHFask, USDCHFask);
  GlobalVariableSet(strNZDUSDask, NZDUSDask);
  GlobalVariableSet(strAUDJPYask, AUDJPYask);
  GlobalVariableSet(strEURJPYask, EURJPYask);

  GlobalVariableSet(strEURUSDbid, EURUSDbid);
  GlobalVariableSet(strUSDJPYbid, USDJPYbid);
  GlobalVariableSet(strAUDUSDbid, AUDUSDbid);
  GlobalVariableSet(strNZDJPYbid, NZDJPYbid);
  GlobalVariableSet(strGBPCHFbid, GBPCHFbid);
  GlobalVariableSet(strCHFJPYbid, CHFJPYbid);
  GlobalVariableSet(strEURCHFbid, EURCHFbid);

  return; //era return(0);
}

//+------------------------------------------------------------------+
//|                       Open phantom trades.                       |
//+------------------------------------------------------------------+

void OpenPhantomTrades()
{
  if (GBPUSDask == 0.0 && EURGBPask == 0.0 &&
      GBPJPYask == 0.0 && USDCHFask == 0.0 &&
      NZDUSDask == 0.0 && AUDJPYask == 0.0 &&
      EURJPYask == 0.0 &&
      
      EURUSDbid == 0.0 && USDJPYbid == 0.0 &&
      AUDUSDbid == 0.0 && NZDJPYbid == 0.0 &&
      GBPCHFbid == 0.0 && CHFJPYbid == 0.0 &&
      EURCHFbid == 0.0)
  {
    GBPUSDask = SymbolInfoDouble(GBPUSDsym, SYMBOL_ASK); //era GBPUSDask = MarketInfo(GBPUSDsym, MODE_ASK);
    EURGBPask = SymbolInfoDouble(EURGBPsym, SYMBOL_ASK); //era EURGBPask = MarketInfo(EURGBPsym, MODE_ASK);
    GBPJPYask = SymbolInfoDouble(GBPJPYsym, SYMBOL_ASK); //era GBPJPYask = MarketInfo(GBPJPYsym, MODE_ASK);
    USDCHFask = SymbolInfoDouble(USDCHFsym, SYMBOL_ASK); //era USDCHFask = MarketInfo(USDCHFsym, MODE_ASK);
    NZDUSDask = SymbolInfoDouble(NZDUSDsym, SYMBOL_ASK); //era NZDUSDask = MarketInfo(NZDUSDsym, MODE_ASK);
    AUDJPYask = SymbolInfoDouble(AUDJPYsym, SYMBOL_ASK); //era AUDJPYask = MarketInfo(AUDJPYsym, MODE_ASK);
    EURJPYask = SymbolInfoDouble(EURJPYsym, SYMBOL_ASK); //era EURJPYask = MarketInfo(EURJPYsym, MODE_ASK);

    EURUSDbid = SymbolInfoDouble(EURUSDsym, SYMBOL_BID); //era EURUSDbid = MarketInfo(EURUSDsym, MODE_BID);
    USDJPYbid = SymbolInfoDouble(USDJPYsym, SYMBOL_BID); //era USDJPYbid = MarketInfo(USDJPYsym, MODE_BID); 
    AUDUSDbid = SymbolInfoDouble(AUDUSDsym, SYMBOL_BID); //era AUDUSDbid = MarketInfo(AUDUSDsym, MODE_BID); 
    NZDJPYbid = SymbolInfoDouble(NZDJPYsym, SYMBOL_BID); //era NZDJPYbid = MarketInfo(NZDJPYsym, MODE_BID); 
    GBPCHFbid = SymbolInfoDouble(GBPCHFsym, SYMBOL_BID); //era GBPCHFbid = MarketInfo(GBPCHFsym, MODE_BID); 
    CHFJPYbid = SymbolInfoDouble(CHFJPYsym, SYMBOL_BID); //era CHFJPYbid = MarketInfo(CHFJPYsym, MODE_BID); 
    EURCHFbid = SymbolInfoDouble(EURCHFsym, SYMBOL_BID); //era EURCHFbid = MarketInfo(EURCHFsym, MODE_BID);

    GlobalVariableSet(strGBPUSDask, GBPUSDask);
    GlobalVariableSet(strEURGBPask, EURGBPask);
    GlobalVariableSet(strGBPJPYask, GBPJPYask);
    GlobalVariableSet(strUSDCHFask, USDCHFask);
    GlobalVariableSet(strNZDUSDask, NZDUSDask);
    GlobalVariableSet(strAUDJPYask, AUDJPYask);
    GlobalVariableSet(strEURJPYask, EURJPYask);

    GlobalVariableSet(strEURUSDbid, EURUSDbid);
    GlobalVariableSet(strUSDJPYbid, USDJPYbid);
    GlobalVariableSet(strAUDUSDbid, AUDUSDbid);
    GlobalVariableSet(strNZDJPYbid, NZDJPYbid);
    GlobalVariableSet(strGBPCHFbid, GBPCHFbid);
    GlobalVariableSet(strCHFJPYbid, CHFJPYbid);
    GlobalVariableSet(strEURCHFbid, EURCHFbid);
  }

  return; // era return(0);
}

//+------------------------------------------------------------------+
//|             Set the account type and minimum equity.             |
//+------------------------------------------------------------------+

void SetAccountType()
{
  if (AccountTypeVal == MiniSize || StringLen(Symbol()) == 7)
  {
    AccountIsMini     = true;
    AccountTypeString = "MINI";
  }
  else if (AccountTypeVal == StdSize || StringLen(Symbol()) == 6)
  {
    AccountIsMini     = false;
    AccountTypeString = "STANDARD";
  }
  
  return; // era return(0);
}

//+------------------------------------------------------------------+
//|                 Set the lot size for any trades.                 |
//+------------------------------------------------------------------+

void SetLotSize()
{
  Investment = MaxMarginToUse / 100.0;

  // The symbol the EA is attached to should be good enough for the purposes of LOTSIZE
  UseLots = valLastFlatEq * Investment * (Leverage / SymbolInfoDouble(Symbol(), SYMBOL_TRADE_CONTRACT_SIZE));
  // era UseLots = StrToDouble(DoubleToStr(UseLots/LotStep,0))*LotStep;
  UseLots = NormalizeDouble(UseLots,2); // era UseLots = StrToDouble(DoubleToStr(UseLots,2));

  if (UseSmallestLot || UseLots < MinLots) UseLots = MinLots;
  if (UseLots > MaxLots) UseLots = MaxLots;

  return; // era return(0);
}

//+------------------------------------------------------------------+
//|  Do some accounting -- count the profit/loss, the open trades,   |
//|  and the number of open lots.                                    |
//+------------------------------------------------------------------+

void Count_PL_Trades_Lots()
{
  NumBuys            =    0;
  NumSells           =    0;
  TotalTrades        =    0;
  EAPL               =    0.0;
  BuyPL              =    0.0;
  SellPL             =    0.0;
  TotalLots          =    0.0;
  BuyLots            =    0.0;
  SellLots           =    0.0;

  GBPUSDbuys = 0; GBPUSDsells = 0;
  EURGBPbuys = 0; EURGBPsells = 0;
  GBPJPYbuys = 0; GBPJPYsells = 0;
  USDCHFbuys = 0; USDCHFsells = 0;
  NZDUSDbuys = 0; NZDUSDsells = 0;
  AUDJPYbuys = 0; AUDJPYsells = 0;
  EURJPYbuys = 0; EURJPYsells = 0;

  EURUSDbuys = 0; EURUSDsells = 0;
  USDJPYbuys = 0; USDJPYsells = 0;
  AUDUSDbuys = 0; AUDUSDsells = 0;
  NZDJPYbuys = 0; NZDJPYsells = 0;
  GBPCHFbuys = 0; GBPCHFsells = 0;
  CHFJPYbuys = 0; CHFJPYsells = 0;
  EURCHFbuys = 0; EURCHFsells = 0;

  BiggestWinnerPL = 0.0;
  BiggestWinnerLots = 0.0;
  BiggestWinnerTicket = 0;
  BiggestWinnerType = "";
  BiggestWinnerSymbol = "";

  BiggestLoserPL = 0.0;
  BiggestLoserLots = 0.0;
  BiggestLoserTicket = 0;
  BiggestLoserType = "";
  BiggestLoserSymbol = "";

  for (i = 0; i < PositionsTotal(); i++)
  {
      PositionGetSymbol(i);
    //SelectByTicket(OrderGetTicket(i)); // era OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
    

//    if(OrderGetInteger(ORDER_MAGIC) == MagicNumber) //era if (OrderMagicNumber() == MagicNumber)
//    {
      PairOverallPL = PositionGetDouble(POSITION_PROFIT) + PositionGetDouble(POSITION_SWAP) + PositionGetDouble(POSITION_COMMISSION);
      EAPL  += PairOverallPL;
      TotalLots += PositionGetDouble(POSITION_VOLUME);

      if (PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY)
      {
        BuyPL += PairOverallPL;

        if (PairOverallPL > BiggestWinnerPL)
        {
          BiggestWinnerPL     = PairOverallPL;
          BiggestWinnerLots   = PositionGetDouble(POSITION_VOLUME);
          BiggestWinnerTicket = 0;
          BiggestWinnerType   = "BUY";
          BiggestWinnerSymbol = PositionGetString(POSITION_SYMBOL);
        }

        if (PairOverallPL < BiggestLoserPL)
        {
          BiggestLoserPL     = PairOverallPL;
          BiggestLoserLots   = PositionGetDouble(POSITION_VOLUME);
          BiggestLoserTicket = 0;
          BiggestLoserType   = "BUY";
          BiggestLoserSymbol = PositionGetString(POSITION_SYMBOL);
        }

        BuyLots += PositionGetDouble(POSITION_VOLUME);

        NumBuys++;

        if (PositionGetString(POSITION_SYMBOL) == GBPUSDsym) { GBPUSDbuys++; GBPUSDPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == EURGBPsym) { EURGBPbuys++; EURGBPPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == GBPJPYsym) { GBPJPYbuys++; GBPJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == USDCHFsym) { USDCHFbuys++; USDCHFPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == NZDUSDsym) { NZDUSDbuys++; NZDUSDPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == AUDJPYsym) { AUDJPYbuys++; AUDJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == EURJPYsym) { EURJPYbuys++; EURJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == EURUSDsym) { EURUSDbuys++; EURUSDPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == USDJPYsym) { USDJPYbuys++; USDJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == AUDUSDsym) { AUDUSDbuys++; AUDUSDPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == NZDJPYsym) { NZDJPYbuys++; NZDJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == GBPCHFsym) { GBPCHFbuys++; GBPCHFPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == CHFJPYsym) { CHFJPYbuys++; CHFJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == EURCHFsym) { EURCHFbuys++; EURCHFPL = PairOverallPL; }
      }

      if (PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_SELL)
      {
        SellPL += PairOverallPL;

        if (PairOverallPL> BiggestWinnerPL)
        {
          BiggestWinnerPL     = PairOverallPL;
          BiggestWinnerLots   = PositionGetDouble(POSITION_VOLUME);
          BiggestWinnerTicket = 0;
          BiggestWinnerType   = "SELL";
          BiggestWinnerSymbol = PositionGetString(POSITION_SYMBOL);
        }

        if (PairOverallPL< BiggestLoserPL)
        {
          BiggestLoserPL     = PairOverallPL;
          BiggestLoserLots   = PositionGetDouble(POSITION_VOLUME);
          BiggestLoserTicket = 0;
          BiggestLoserType   = "SELL";
          BiggestLoserSymbol = PositionGetString(POSITION_SYMBOL);
        }

        SellLots += PositionGetDouble(POSITION_VOLUME);

        NumSells++;

        if (PositionGetString(POSITION_SYMBOL) == GBPUSDsym) { GBPUSDsells++; GBPUSDPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == EURGBPsym) { EURGBPsells++; EURGBPPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == GBPJPYsym) { GBPJPYsells++; GBPJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == USDCHFsym) { USDCHFsells++; USDCHFPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == NZDUSDsym) { NZDUSDsells++; NZDUSDPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == AUDJPYsym) { AUDJPYsells++; AUDJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == EURJPYsym) { EURJPYsells++; EURJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == EURUSDsym) { EURUSDsells++; EURUSDPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == USDJPYsym) { USDJPYsells++; USDJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == AUDUSDsym) { AUDUSDsells++; AUDUSDPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == NZDJPYsym) { NZDJPYsells++; NZDJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == GBPCHFsym) { GBPCHFsells++; GBPCHFPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == CHFJPYsym) { CHFJPYsells++; CHFJPYPL = PairOverallPL; }
        else
        if (PositionGetString(POSITION_SYMBOL) == EURCHFsym) { EURCHFsells++; EURCHFPL = PairOverallPL; }
      }
//    }
  }
  TotalTrades = NumBuys+NumSells;

  return; // era return(0);
}

//+------------------------------------------------------------------+
//|         Set the dollar profit and trim target values.            |
//+------------------------------------------------------------------+

void SetExitTargets()
{
  DollarProfitTarget = StringToDouble(DoubleToString(valLastFlatEq*ProfitPct/100.0,2));

  if (AccountIsMini)
  {
    if (DollarProfitTarget > MaxMiniProfit) DollarProfitTarget = MaxMiniProfit;
  }
  else
  {
    if (DollarProfitTarget > MaxStdProfit) DollarProfitTarget = MaxStdProfit;
  }

  return; // era return(0);
}

//+------------------------------------------------------------------+
//|                 Close all of the open trades.                    |
//+------------------------------------------------------------------+

void CloseAllTradesIfNeeded()
{
  if (BidsHaveChanged && CloseAll)
  {
        for(i = 0; i < 6; i++) mytrade.PositionClose(SymbolArray[i], -1);
/*    for(i = 0; i < OrdersTotal(); i++)
    {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);

      if (OrderMagicNumber() == MagicNumber)
      {
        if (OrderType() == OP_BUY)
        {
          Comment("In full closure mode.  Closing a ", OrderSymbol(), " buy trade...");
          Print("In full closure mode.  Closing a ", OrderSymbol(), " buy trade...");
          OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(), MODE_BID),Slippage,White);
          Sleep(1000);
        }    
        else
        if (OrderType() == OP_SELL)
        {
          Comment("In full closure mode.  Closing a ", OrderSymbol(), " sell trade...");
          Print("In full closure mode.  Closing a ", OrderSymbol(), " sell trade...");
          OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(), MODE_ASK),Slippage,White);
          Sleep(1000);
        }
      }
    }
*/  }
  
  return; // era return(0);
}

//+------------------------------------------------------------------+
//|                 Clear all symbol-added marks.                    |
//+------------------------------------------------------------------+

void ClearSymbolMarks()
{
  valGBPUSDadded = false;
  GlobalVariableSet(GBPUSDadded, valGBPUSDadded);

  valEURGBPadded = false;
  GlobalVariableSet(EURGBPadded, valEURGBPadded);

  valGBPJPYadded = false;
  GlobalVariableSet(GBPJPYadded, valGBPJPYadded);

  valUSDCHFadded = false;
  GlobalVariableSet(USDCHFadded, valUSDCHFadded);

  valNZDUSDadded = false;
  GlobalVariableSet(NZDUSDadded, valNZDUSDadded);

  valAUDJPYadded = false;
  GlobalVariableSet(AUDJPYadded, valAUDJPYadded);

  valEURJPYadded = false;
  GlobalVariableSet(EURJPYadded, valEURJPYadded);

  valEURUSDadded = false;
  GlobalVariableSet(EURUSDadded, valEURUSDadded);

  valUSDJPYadded = false;
  GlobalVariableSet(USDJPYadded, valUSDJPYadded);

  valAUDUSDadded = false;
  GlobalVariableSet(AUDUSDadded, valAUDUSDadded);

  valNZDJPYadded = false;
  GlobalVariableSet(NZDJPYadded, valNZDJPYadded);

  valGBPCHFadded = false;
  GlobalVariableSet(GBPCHFadded, valGBPCHFadded);

  valCHFJPYadded = false;
  GlobalVariableSet(CHFJPYadded, valCHFJPYadded);

  valEURCHFadded = false;
  GlobalVariableSet(EURCHFadded, valEURCHFadded);

  return; // era return(0);
}

//+------------------------------------------------------------------+
//|                 Mark the given currency pair as added.           |
//+------------------------------------------------------------------+

void MarkSymbolAdded(string theSymbol)
{
  if (theSymbol == GBPUSDsym)
  {
    valGBPUSDadded = true;
    GlobalVariableSet(GBPUSDadded, valGBPUSDadded);
  }
  else
  if (theSymbol == EURGBPsym)
  {
    valEURGBPadded = true;
    GlobalVariableSet(EURGBPadded, valEURGBPadded);
  }
  else
  if (theSymbol == GBPJPYsym)
  {
    valGBPJPYadded = true;
    GlobalVariableSet(GBPJPYadded, valGBPJPYadded);
  }
  else
  if (theSymbol == USDCHFsym)
  {
    valUSDCHFadded = true;
    GlobalVariableSet(USDCHFadded, valUSDCHFadded);
  }
  else
  if (theSymbol == NZDUSDsym)
  {
    valNZDUSDadded = true;
    GlobalVariableSet(NZDUSDadded, valNZDUSDadded);
  }
  else
  if (theSymbol == AUDJPYsym)
  {
    valAUDJPYadded = true;
    GlobalVariableSet(AUDJPYadded, valAUDJPYadded);
  }
  else
  if (theSymbol == EURJPYsym)
  {
    valEURJPYadded = true;
    GlobalVariableSet(EURJPYadded, valEURJPYadded);
  }
  else
  if (theSymbol == EURUSDsym)
  {
    valEURUSDadded = true;
    GlobalVariableSet(EURUSDadded, valEURUSDadded);
  }
  else
  if (theSymbol == USDJPYsym)
  {
    valUSDJPYadded = true;
    GlobalVariableSet(USDJPYadded, valUSDJPYadded);
  }
  else
  if (theSymbol == AUDUSDsym)
  {
    valAUDUSDadded = true;
    GlobalVariableSet(AUDUSDadded, valAUDUSDadded);
  }
  else
  if (theSymbol == NZDJPYsym)
  {
    valNZDJPYadded = true;
    GlobalVariableSet(NZDJPYadded, valNZDJPYadded);
  }
  else
  if (theSymbol == GBPCHFsym)
  {
    valGBPCHFadded = true;
    GlobalVariableSet(GBPCHFadded, valGBPCHFadded);
  }
  else
  if (theSymbol == CHFJPYsym)
  {
    valCHFJPYadded = true;
    GlobalVariableSet(CHFJPYadded, valCHFJPYadded);
  }
  else
  if (theSymbol == EURCHFsym)
  {
    valEURCHFadded = true;
    GlobalVariableSet(EURCHFadded, valEURCHFadded);
  }

  return; // era return(0);
}

//+------------------------------------------------------------------+
//|          Check if the given currency is marked as added.         |
//+------------------------------------------------------------------+

bool SymbolMarked(string theSymbol)
{
  if (theSymbol == GBPUSDsym && valGBPUSDadded)
  {
    return(true);
  }
  else
  if (theSymbol == EURGBPsym && valEURGBPadded)
  {
    return(true);
  }
  else
  if (theSymbol == GBPJPYsym && valGBPJPYadded)
  {
    return(true);
  }
  else
  if (theSymbol == USDCHFsym && valUSDCHFadded)
  {
    return(true);
  }
  else
  if (theSymbol == NZDUSDsym && valNZDUSDadded)
  {
    return(true);
  }
  else
  if (theSymbol == AUDJPYsym && valAUDJPYadded)
  {
    return(true);
  }
  else
  if (theSymbol == EURJPYsym && valEURJPYadded)
  {
    return(true);
  }
  else
  if (theSymbol == EURUSDsym && valEURUSDadded)
  {
    return(true);
  }
  else
  if (theSymbol == USDJPYsym && valUSDJPYadded)
  {
    return(true);
  }
  else
  if (theSymbol == AUDUSDsym && valAUDUSDadded)
  {
    return(true);
  }
  else
  if (theSymbol == NZDJPYsym && valNZDJPYadded)
  {
    return(true);
  }
  else
  if (theSymbol == GBPCHFsym && valGBPCHFadded)
  {
    return(true);
  }
  else
  if (theSymbol == CHFJPYsym && valCHFJPYadded)
  {
    return(true);
  }
  else
  if (theSymbol == EURCHFsym && valEURCHFadded)
  {
    return(true);
  }

  return(false);
}

//+------------------------------------------------------------------+
//|     Correct problem of brokers using 3 or 5 digit pricing        |
//+------------------------------------------------------------------+

double SetPoint(string theSymbol)
{
  double mPoint, mDigits;
   
  mDigits = (int)SymbolInfoInteger(theSymbol, SYMBOL_DIGITS);

  if (mDigits < 4)
    mPoint = 0.01;
  else
    mPoint = 0.0001;

  return(mPoint);
}

//+------------------------------------------------------------------+
//|     Correct problem of brokers using 3 or 5 digit pricing        |
//+------------------------------------------------------------------+

double SetTickValue(string theSymbol, double thePoint)
{
  double mTickValue;
   
  if (thePoint/10.0 == SymbolInfoDouble(theSymbol, SYMBOL_POINT))
  {
    mTickValue = SymbolInfoDouble(theSymbol, SYMBOL_TRADE_TICK_VALUE)*10.0;
  }
  else // if (thePoint == MarketInfo(theSymbol, MODE_POINT))
  {
    mTickValue = SymbolInfoDouble(theSymbol, SYMBOL_TRADE_TICK_VALUE);
  }

  return(mTickValue);
}

//+------------------------------------------------------------------+
//|                Set the value of Point for each pair.             |
//+------------------------------------------------------------------+

void SetPointsAndTickvalues()
{
  GBPUSDpoint = SetPoint(GBPUSDsym);
  EURGBPpoint = SetPoint(EURGBPsym);
  GBPJPYpoint = SetPoint(GBPJPYsym);
  USDCHFpoint = SetPoint(USDCHFsym);
  NZDUSDpoint = SetPoint(NZDUSDsym);
  AUDJPYpoint = SetPoint(AUDJPYsym);
  EURJPYpoint = SetPoint(EURJPYsym);

  EURUSDpoint = SetPoint(EURUSDsym);
  USDJPYpoint = SetPoint(USDJPYsym);
  AUDUSDpoint = SetPoint(AUDUSDsym);
  NZDJPYpoint = SetPoint(NZDJPYsym);
  GBPCHFpoint = SetPoint(GBPCHFsym);
  CHFJPYpoint = SetPoint(CHFJPYsym);
  EURCHFpoint = SetPoint(EURCHFsym);

  GBPUSDtickval = SetTickValue(GBPUSDsym, GBPUSDpoint);
  TickvalArray[GBPUSDidx] = GBPUSDtickval;

  EURGBPtickval = SetTickValue(EURGBPsym, EURGBPpoint);
  TickvalArray[EURGBPidx] = EURGBPtickval;

  GBPJPYtickval = SetTickValue(GBPJPYsym, GBPJPYpoint);
  TickvalArray[GBPJPYidx] = GBPJPYtickval;

  USDCHFtickval = SetTickValue(USDCHFsym, USDCHFpoint);
  TickvalArray[USDCHFidx] = USDCHFtickval;

  NZDUSDtickval = SetTickValue(NZDUSDsym, NZDUSDpoint);
  TickvalArray[NZDUSDidx] = NZDUSDtickval;

  AUDJPYtickval = SetTickValue(AUDJPYsym, AUDJPYpoint);
  TickvalArray[AUDJPYidx] = AUDJPYtickval;

  EURJPYtickval = SetTickValue(EURJPYsym, EURJPYpoint);
  TickvalArray[EURJPYidx] = EURJPYtickval;

  EURUSDtickval = SetTickValue(EURUSDsym, EURUSDpoint);
  TickvalArray[EURUSDidx] = EURUSDtickval;

  USDJPYtickval = SetTickValue(USDJPYsym, USDJPYpoint);
  TickvalArray[USDJPYidx] = USDJPYtickval;

  AUDUSDtickval = SetTickValue(AUDUSDsym, AUDUSDpoint);
  TickvalArray[AUDUSDidx] = AUDUSDtickval;

  NZDJPYtickval = SetTickValue(NZDJPYsym, NZDJPYpoint);
  TickvalArray[NZDJPYidx] = NZDJPYtickval;

  GBPCHFtickval = SetTickValue(GBPCHFsym, GBPCHFpoint);
  TickvalArray[GBPCHFidx] = GBPCHFtickval;

  CHFJPYtickval = SetTickValue(CHFJPYsym, CHFJPYpoint);
  TickvalArray[CHFJPYidx] = CHFJPYtickval;

  EURCHFtickval = SetTickValue(EURCHFsym, EURCHFpoint);
  TickvalArray[EURCHFidx] = EURCHFtickval;

  return;
}

//+------------------------------------------------------------------+
//|                 Calculate PL for phantom trades.                 |
//+------------------------------------------------------------------+

void CalcPhantomPL()
{
  SetPointsAndTickvalues();
  
  if (GBPUSDpoint != 0.0)
  {
    GBPUSD_PL = (SymbolInfoDouble(GBPUSDsym, SYMBOL_BID) - GBPUSDask) / GBPUSDpoint;
    GBPUSD_PL *= TickvalArray[GBPUSDidx]*UseLots;
    PL_Array[GBPUSDidx] = GBPUSD_PL;
  }

  if (EURGBPpoint != 0.0)
  {
    EURGBP_PL = (SymbolInfoDouble(EURGBPsym, SYMBOL_BID) - EURGBPask) / EURGBPpoint;
    EURGBP_PL *= TickvalArray[EURGBPidx]*UseLots;
    PL_Array[EURGBPidx] = EURGBP_PL;
  }
  
  if (GBPJPYpoint != 0.0)
  {
    GBPJPY_PL = (SymbolInfoDouble(GBPJPYsym, SYMBOL_BID) - GBPJPYask) / GBPJPYpoint;
    GBPJPY_PL *= TickvalArray[GBPJPYidx]*UseLots;
    PL_Array[GBPJPYidx] = GBPJPY_PL;
  }

  if (USDCHFpoint != 0.0)
  {
    USDCHF_PL = (SymbolInfoDouble(USDCHFsym, SYMBOL_BID) - USDCHFask) / USDCHFpoint;
    USDCHF_PL *= TickvalArray[USDCHFidx]*UseLots;
    PL_Array[USDCHFidx] = USDCHF_PL;
  }

  if (NZDUSDpoint != 0.0)
  {
    NZDUSD_PL = (SymbolInfoDouble(NZDUSDsym, SYMBOL_BID) - NZDUSDask) / NZDUSDpoint;
    NZDUSD_PL *= TickvalArray[NZDUSDidx]*UseLots;
    PL_Array[NZDUSDidx] = NZDUSD_PL;
  }

  if (AUDJPYpoint != 0.0)
  {
    AUDJPY_PL = (SymbolInfoDouble(AUDJPYsym, SYMBOL_BID) - AUDJPYask) / AUDJPYpoint;
    AUDJPY_PL *= TickvalArray[AUDJPYidx]*UseLots;
    PL_Array[AUDJPYidx] = AUDJPY_PL;
  }

  if (EURJPYpoint != 0.0)
  {
    EURJPY_PL = (SymbolInfoDouble(EURJPYsym, SYMBOL_BID) - EURJPYask) / EURJPYpoint;
    EURJPY_PL *= TickvalArray[EURJPYidx]*UseLots;
    PL_Array[EURJPYidx] = EURJPY_PL;
  }

  phantomBuyPL = GBPUSD_PL + EURGBP_PL + GBPJPY_PL + USDCHF_PL + NZDUSD_PL + AUDJPY_PL + EURJPY_PL;

  if (EURUSDpoint != 0.0)
  {
    EURUSD_PL = (EURUSDbid - SymbolInfoDouble(EURUSDsym, SYMBOL_ASK)) / EURUSDpoint;
    EURUSD_PL *= TickvalArray[EURUSDidx]*UseLots;
    PL_Array[EURUSDidx] = EURUSD_PL;
  }

  if (USDJPYpoint != 0.0)
  {
    USDJPY_PL = (USDJPYbid - SymbolInfoDouble(USDJPYsym, SYMBOL_ASK)) / USDJPYpoint;
    USDJPY_PL *= TickvalArray[USDJPYidx]*UseLots;
    PL_Array[USDJPYidx] = USDJPY_PL;
  }

  if (AUDUSDpoint != 0.0)
  {
    AUDUSD_PL = (AUDUSDbid - SymbolInfoDouble(AUDUSDsym, SYMBOL_ASK)) / AUDUSDpoint;
    AUDUSD_PL *= TickvalArray[AUDUSDidx]*UseLots;
    PL_Array[AUDUSDidx] = AUDUSD_PL;
  }

  if (NZDJPYpoint != 0.0)
  {
    NZDJPY_PL = (NZDJPYbid - SymbolInfoDouble(NZDJPYsym, SYMBOL_ASK)) / NZDJPYpoint;
    NZDJPY_PL *= TickvalArray[NZDJPYidx]*UseLots;
    PL_Array[NZDJPYidx] = NZDJPY_PL;
  }

  if (GBPCHFpoint != 0.0)
  {
    GBPCHF_PL = (GBPCHFbid - SymbolInfoDouble(GBPCHFsym, SYMBOL_ASK)) / GBPCHFpoint;
    GBPCHF_PL *= TickvalArray[GBPCHFidx]*UseLots;
    PL_Array[GBPCHFidx] = GBPCHF_PL;
  }

  if (CHFJPYpoint != 0.0)
  {
    CHFJPY_PL = (CHFJPYbid - SymbolInfoDouble(CHFJPYsym, SYMBOL_ASK)) / CHFJPYpoint;
    CHFJPY_PL *= TickvalArray[CHFJPYidx]*UseLots;
    PL_Array[CHFJPYidx] = CHFJPY_PL;
  }

  if (EURCHFpoint != 0.0)
  {
    EURCHF_PL = (EURCHFbid - SymbolInfoDouble(EURCHFsym, SYMBOL_ASK)) / EURCHFpoint;
    EURCHF_PL *= TickvalArray[EURCHFidx]*UseLots;
    PL_Array[EURCHFidx] = EURCHF_PL;
  }

  phantomSellPL = EURUSD_PL + USDJPY_PL + AUDUSD_PL + NZDJPY_PL + GBPCHF_PL + CHFJPY_PL + EURCHF_PL;
  phantomEAPL = phantomBuyPL + phantomSellPL;

  // Determine biggest phantom winner.

  BiggestPhantomWinnerPL = 0.0;
  BiggestPhantomWinnerType = "";
  BiggestPhantomWinnerSymbol = "";

  if (GBPUSD_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = GBPUSD_PL;
    BiggestPhantomWinnerType   = "BUY";
    BiggestPhantomWinnerSymbol = GBPUSDsym;
  }

  if (EURGBP_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = EURGBP_PL;
    BiggestPhantomWinnerType   = "BUY";
    BiggestPhantomWinnerSymbol = EURGBPsym;
  }
  if (GBPJPY_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = GBPJPY_PL;
    BiggestPhantomWinnerType   = "BUY";
    BiggestPhantomWinnerSymbol = GBPJPYsym;
  }
  if (USDCHF_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = USDCHF_PL;
    BiggestPhantomWinnerType   = "BUY";
    BiggestPhantomWinnerSymbol = USDCHFsym;
  }
  if (NZDUSD_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = NZDUSD_PL;
    BiggestPhantomWinnerType   = "BUY";
    BiggestPhantomWinnerSymbol = NZDUSDsym;
  }
  if (AUDJPY_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = AUDJPY_PL;
    BiggestPhantomWinnerType   = "BUY";
    BiggestPhantomWinnerSymbol = AUDJPYsym;
  }
  if (EURJPY_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = EURJPY_PL;
    BiggestPhantomWinnerType   = "BUY";
    BiggestPhantomWinnerSymbol = EURJPYsym;
  }


  if (EURUSD_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = EURUSD_PL;
    BiggestPhantomWinnerType   = "SELL";
    BiggestPhantomWinnerSymbol = EURUSDsym;
  }
  if (USDJPY_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = USDJPY_PL;
    BiggestPhantomWinnerType   = "SELL";
    BiggestPhantomWinnerSymbol = USDJPYsym;
  }
  if (AUDUSD_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = AUDUSD_PL;
    BiggestPhantomWinnerType   = "SELL";
    BiggestPhantomWinnerSymbol = AUDUSDsym;
  }
  if (NZDJPY_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = NZDJPY_PL;
    BiggestPhantomWinnerType   = "SELL";
    BiggestPhantomWinnerSymbol = NZDJPYsym;
  }
  if (GBPCHF_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = GBPCHF_PL;
    BiggestPhantomWinnerType   = "SELL";
    BiggestPhantomWinnerSymbol = GBPCHFsym;
  }
  if (CHFJPY_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = CHFJPY_PL;
    BiggestPhantomWinnerType   = "SELL";
    BiggestPhantomWinnerSymbol = CHFJPYsym;
  }
  if (EURCHF_PL > BiggestPhantomWinnerPL)
  {
    BiggestPhantomWinnerPL     = EURCHF_PL;
    BiggestPhantomWinnerType   = "SELL";
    BiggestPhantomWinnerSymbol = EURCHFsym;
  }

  return;
}

//+------------------------------------------------------------------+
//|                     Add to winning trades.                       |
//+------------------------------------------------------------------+

void AddIfNeeded()
{
  MqlDateTime CurrentTime;
  
  TimeToStruct(TimeCurrent(),CurrentTime);
  
  if (!PhantomsReset &&
      MathMod(CurrentTime.min,ResetHourInterval) == 0.0 && 
      CurrentTime.min == 1)
  {
    ResetPhantoms();
    PhantomsReset = true;
  }

  if (!SymbolMarksCleared &&
      CurrentTime.min != 0)
  {
    ClearSymbolMarks();
    SymbolMarksCleared = true;
  }

  if (BidsHaveChanged &&
      CurrentTime.min == 0 &&
     !CloseAll)
  {
    PhantomsReset      = false;
    SymbolMarksCleared = false;

    for (i = GBPUSDidx; i <= EURCHFidx; i++)
    {
      if (SymbolArray[i] == BiggestPhantomWinnerSymbol &&
           (SymbolArray[i] == EURCHFsym || SymbolArray[i] == EURJPYsym || SymbolArray[i] == CHFJPYsym ||
            SymbolArray[i] == GBPCHFsym || SymbolArray[i] == GBPUSDsym || SymbolArray[i] == USDCHFsym) &&
         !SymbolMarked(SymbolArray[i]))
      {
        if (i < 7)
        {
          Comment("Adding a " + SymbolArray[i] + " buy trade...");
          Print("Adding a " + SymbolArray[i] + " long.");
          OrderSendResult = mytrade.PositionOpen(SymbolArray[i], ORDER_TYPE_BUY, UseLots,
                              SymbolInfoDouble(SymbolArray[i], SYMBOL_ASK),0,0);
          if (OrderSendResult > 0) MarkSymbolAdded(SymbolArray[i]);
          Sleep(1000);
        }
        else
        // if (i >= 7)
        {
          Comment("Adding a " + SymbolArray[i] + " sell trade...");
          Print("Adding a " + SymbolArray[i] + " short.");
          OrderSendResult = mytrade.PositionOpen(SymbolArray[i], ORDER_TYPE_SELL, UseLots,
                              SymbolInfoDouble(SymbolArray[i], SYMBOL_BID),0,0);
          if (OrderSendResult > 0) MarkSymbolAdded(SymbolArray[i]);
          Sleep(1000);
        }
      }
    } 
  }

  return;
}

//+------------------------------------------------------------------+
//|   Keep track of lowest margin level and largest floating loss.   |
//+------------------------------------------------------------------+

void ExamineMarginAndFloat()
{
  if (AccountInfoDouble(ACCOUNT_MARGIN) > 0.0)
  {
    PercentMarginLevel = StringToDouble(DoubleToString(100 * AccountInfoDouble(ACCOUNT_EQUITY) / AccountInfoDouble(ACCOUNT_MARGIN), 2));

    if (PercentMarginLevel < valLowestMarginLevel)
    {
      valLowestMarginLevel = PercentMarginLevel;
      GlobalVariableSet(LowestMarginLevel, valLowestMarginLevel);
    }
  }

  if (EAPL < valLargestFloatingLoss)
  {
    valLargestFloatingLoss = StringToDouble(DoubleToString(EAPL, 2));
    GlobalVariableSet(LargestFloatingLoss, valLargestFloatingLoss);
  }

  if ((valLargestFloatingLoss / valLastFlatEq) * 100.0 < valMaxFloatingDrawdown)
  {
    valMaxFloatingDrawdown = StringToDouble(DoubleToString((valLargestFloatingLoss / valLastFlatEq) * 100.0, 2));
    GlobalVariableSet(MaxFloatingDrawdown, valMaxFloatingDrawdown);
  }

  return;
}

//+------------------------------------------------------------------+
//|                      Print Info to Chart                         |
//+------------------------------------------------------------------+

void PrintChartComments()
{
  CommentString = 
        "\nSTRATEGY: Add to winning trades once per hour." +
        "\n\nREMINDER: Be FLAT (all trades closed) before depositing/withdrawing funds." +
        "\n\nAccount Type: " + AccountTypeString + " Account,   Leverage: " + DoubleToString(Leverage,0) + ":1" +
        ",   Margin %: " + DoubleToString(PercentMarginLevel,2) + "%,   Lowest Margin %: " + DoubleToString(valLowestMarginLevel,2) + "%" +
        "\n\nLargest Floating Loss: $" + DoubleToString(valLargestFloatingLoss,2) + ",  Max Floating Drawdown %: " + DoubleToString(valMaxFloatingDrawdown,2) + "%" +
        "\n\nBalance: $" + DoubleToString(AccountInfoDouble(ACCOUNT_BALANCE),2) + ",   Equity: $" + DoubleToString(AccountInfoDouble(ACCOUNT_EQUITY),2) +
        "\nLast Flat Balance: $" + DoubleToString(valLastFlatBal,2) + ",   Last Flat Equity: $" + DoubleToString(valLastFlatEq,2) +
        "\n\nMax Margin To Use: " + DoubleToString(MaxMarginToUse,4) + ",   Use Lots: " + DoubleToString(UseLots,2) +
        "\nProfit Percent: " + DoubleToString(ProfitPct,4) + ",   Dollar Profit Target: $" + DoubleToString(DollarProfitTarget,2) + ",   Equity Profit Target: $" + DoubleToString(valLastFlatEq+DollarProfitTarget,2) +
        "\nOpen Buys: " + DoubleToString(NumBuys,0) + ",   Open Sells: " + DoubleToString(NumSells,0) + ",   Total Open Trades: " + DoubleToString(TotalTrades,0) +
        "\n\nGBPUSD Phantom PL: $" + DoubleToString(GBPUSD_PL,2) +
        "\nEURGBP Phantom PL: $" + DoubleToString(EURGBP_PL,2) +
        "\nGBPJPY Phantom PL: $" + DoubleToString(GBPJPY_PL,2) +
        "\nUSDCHF Phantom PL: $" + DoubleToString(USDCHF_PL,2) +
        "\nNZDUSD Phantom PL: $" + DoubleToString(NZDUSD_PL,2) +
        "\nAUDJPY Phantom PL: $" + DoubleToString(AUDJPY_PL,2) +
        "\nEURJPY Phantom PL: $" + DoubleToString(EURJPY_PL,2) +
        "\nEURUSD Phantom PL: $" + DoubleToString(EURUSD_PL,2) +
        "\nUSDJPY Phantom PL: $" + DoubleToString(USDJPY_PL,2) +
        "\nAUDUSD Phantom PL: $" + DoubleToString(AUDUSD_PL,2) +
        "\nNZDJPY Phantom PL: $" + DoubleToString(NZDJPY_PL,2) +
        "\nGBPCHF Phantom PL: $" + DoubleToString(GBPCHF_PL,2) +
        "\nCHFJPY Phantom PL: $" + DoubleToString(CHFJPY_PL,2) +
        "\nEURCHF Phantom PL: $" + DoubleToString(EURCHF_PL,2) +
        "\nPhantom Buy PL: $" + DoubleToString(phantomBuyPL,2) + ",   Phantom Sell PL: $" + DoubleToString(phantomSellPL,2) + "   Overall Phantom PL: $" + DoubleToString(phantomEAPL,2) +
        "\nEA PL: $" + DoubleToString(EAPL,2) + ",   Buy PL: $" + DoubleToString(BuyPL,2) + ",   Sell PL: $" + DoubleToString(SellPL,2) +
        "\n\nBiggest Winner Symbol: " + BiggestWinnerSymbol + ",   Biggest Winner Type: " + BiggestWinnerType + ",  Biggest Winner Ticket: " + DoubleToString(BiggestWinnerTicket,0) + ",   Biggest Winner PL: $" + DoubleToString(BiggestWinnerPL,2) +
        "\nBiggest Loser Symbol: " + BiggestLoserSymbol + ",   Biggest Loser Type: " + BiggestLoserType + ",  Biggest Loser Ticket: " + DoubleToString(BiggestLoserTicket,0) + ",   Biggest Loser PL: $" + DoubleToString(BiggestLoserPL,2);
        
  Comment(CommentString);

  return;
}

